@@ -11,14 +11,14 @@ sum(-logl)
 {
 k<-ncol(x)
 ## perform optimization
-p<-optim(array(0.1,dim=c(1,k)),.poissonlik,y=y,x=x,method="BFGS",hessian=T)
+op<-optim(array(0.1,dim=c(1,k)),.poissonlik,y=y,x=x,method="BFGS",hessian=T)
 ## compute coefficients
-coef<-c(p$par)
+coef<-c(op$par)
 ## degrees of freedom and standard deciation of residuals 
 df <- nrow(x)-ncol(x)
 sigma2 <- sum((y - exp(x%*%coef))^2)/df
 ## compute covariance matrix
-vcov<-solve(p$hessian)
+vcov<-solve(op$hessian)
 colnames(vcov) <- rownames(vcov) <- colnames(x)
 list(coefficients = coef,
 vcov = vcov,
@@ -26,7 +26,7 @@ sigma = sqrt(sigma2),
 df = df)
 }
 
-.poisson.default <- function(x, y, ...)
+.poisson.default <- function(x, y)
 {
 x <- as.matrix(x)
 y <- as.numeric(y)
@@ -39,85 +39,131 @@ est
 }
 
 
-.acpliknc<-function(theta,y){ 
+.acpliknc<-function(theta, y, pp, qq){ 
 n<-nrow(y)
 lambda <- matrix(NA,n,1)
-lambda[1] <- theta[1]+theta[2]*mean(y)+theta[3]*mean(y)
-lambda[2:n] <- filter(theta[1] + theta[2] * y[1:(n-1)], filter = theta[3], init=lambda[1], method = "recursive")
+lambda[1] <- mean(y)
+vylag = Lag(y,1)
+vparlag = c(theta[2])
+if(pp > 1){
+for (i in 1:(pp-1)){ vparlag <- append(vparlag, theta[2+i]) 
+vylag <- cbind(vylag,Lag(y,i+1))
+lambda[i+1] <- mean(y)
+}}
+vfilt = c(theta[2+pp])
+vinit = c(lambda[1])
+if(qq > 1){
+for (i in 1:(qq-1)){ vfilt <- append(vfilt, theta[2+pp+i]) 
+vinit <- append(vinit,lambda[1])
+}}
+lambda[(pp+1):n] <- filter(theta[1] + na.omit(vylag%*%vparlag), filter = vfilt, init = vinit, method = "recursive")
 mu<-lambda
 logl<-(y*log(mu))-mu-lfactorial(y)
 sum(-logl)
 }
 
-.acplikc<-function(theta,y,x){ 
+.acplikc<-function(theta, y, x, pp, qq){ 
 n<-nrow(x)
 k<-ncol(x)
 beta<-theta[1:k]
 lambda <- matrix(NA,n,1)
-lambda[1] <- theta[k+1]+theta[k+2]*mean(y)+theta[k+3]*mean(y)
-lambda[2:n] <- filter(theta[k + 1] + theta[k + 2] * y[1:(n-1)], filter = theta[k + 3], init=lambda[1], method = "recursive")
+lambda[1] <- mean(y)
+vylag = Lag(y,1)
+vparlag = c(theta[k+2])
+if(pp > 1){
+for (i in 1:(pp-1)){ vparlag <- append(vparlag, theta[k+2+i]) 
+vylag <- cbind(vylag,Lag(y,i+1))
+lambda[i+1] <- mean(y)
+}}
+vfilt = c(theta[k+2+pp])
+vinit = c(lambda[1])
+if(qq > 1){
+for (i in 1:(qq-1)){ vfilt <- append(vfilt, theta[k+2+pp+i]) 
+vinit <- append(vinit,lambda[1])
+}}
+lambda[(pp+1):n] <- filter(theta[k + 1] + na.omit(vylag%*%vparlag), filter = vfilt, init = vinit, method = "recursive")
 mu<-exp(x%*%beta)*lambda
 logl<-(y*log(mu))-mu-lfactorial(y)
 sum(-logl)
 }
 
 
-.acpEst <- function(x, y, startval,varopt)
+.acpEst <- function(x, y, p, q, startval, varopt)
 {
 n<-nrow(x)
-
 if(sum(x)==0){
 if(is.null(startval)){
 #arma regression to detect initial values for autoregressive parameters
-ar<-arma(y, order = c(1, 1))
-asval <- matrix(NA,1,3)
-asval[3]<-abs(coef(ar)[2])
-asval[2]<-abs(abs(coef(ar)[1])-asval[3])
-asval[1]<-(mean(y)*abs(1-asval[2]-asval[3]))
+ar<-arma(y, order = c(p, q))
+asval <- matrix(NA,1,1+p+q)
+for (i in 1:p){ asval[1+i] = abs(coef(ar)[i]) }
+for (i in 1:q){ asval[1+p+i] = abs(coef(ar)[p+i]) }
+asval[1]<-(mean(y)*abs(1-asval[2]-asval[1+p+1]))
 ## perform optimization
-p<-optim(c(asval),.acpliknc,y=y,method="BFGS",hessian=varopt)
+op<-optim(c(asval),fn=.acpliknc,gr=NULL,y=y,pp=p,qq=q,method="BFGS",hessian=varopt)
 }
 else{
-p<-optim(startval,.acpliknc,y=y,method="BFGS",hessian=varopt)
+op<-optim(startval,fn=.acpliknc,gr=NULL,y=y,pp=p,qq=q,method="BFGS",hessian=varopt)
 }
 
 ## compute coefficients
-coef<-p$par
+coef<-op$par
 ## compute conditional mean
 lambda <- matrix(NA,n,1)
-lambda[1] <- coef[1]+coef[2]*mean(y)+coef[3]*mean(y)
-for (t in 2:n) {lambda[t] <- coef[1]+coef[2]*y[t-1]+coef[3]*lambda[t-1]}
+lambda[1] <- mean(y)
+vylag = Lag(y,1)
+vparlag = c(coef[2])
+if(p > 1){
+for (i in 1:(p-1)){ vparlag <- append(vparlag, coef[2+i]) 
+vylag <- cbind(vylag,Lag(y,i+1))
+lambda[i+1] <- mean(y)
+}}
+vfilt = c(coef[2+p])
+vinit = c(lambda[1])
+if(q > 1){
+for (i in 1:(q-1)){ vfilt <- append(vfilt, coef[2+p+i]) 
+vinit <- append(vinit,lambda[1])
+}}
+lambda[(p+1):n] <- filter(coef[1] + na.omit(vylag%*%vparlag), filter = vfilt, init = vinit, method = "recursive")
 mu<-lambda
 ## compute standardized residuals
 pres<-(y-mu)/sqrt(mu)
 ## degrees of freedom and standard deviation of residuals 
-df <- nrow(x)-3
+df <- nrow(y)-p-q-1
 sigma2 <- sum(pres^2)/df
-if(is.null(p$hessian))
+if(is.null(op$hessian))
 {
-vcov<- matrix(NA,1,3)
+vcov<- matrix(NA,1,p+q+1)
 ## create vector of parameter names
-namevec <- matrix(NA,1,3)
+namevec <- matrix(NA,1,p+q+1)
 namevec[1]<-"a"
-namevec[2]<-"b"
-namevec[3]<-"c"
+for (i in 1:p){ 
+namevec[1+i]<-paste("b",i)
+}
+for (i in 1:q){ 
+namevec[1+p+i]<-paste("c",i)
+}
 list(coefficients = coef, vcov = vcov,
 df = df)
 }
 else
 {
 ## compute covariance matrix
-vcov<-solve(p$hessian)
+vcov<-solve(op$hessian)
 ## create vector of parameter names
-namevec <- matrix(NA,1,3)
+namevec <- matrix(NA,1,p+q+1)
 namevec[1]<-"a"
-namevec[2]<-"b"
-namevec[3]<-"c"
+for (i in 1:p){ 
+namevec[1+i]<-paste("b",i)
+}
+for (i in 1:q){ 
+namevec[1+p+i]<-paste("c",i)
+}
 colnames(vcov) <- rownames(vcov) <- namevec
 list(coefficients = coef,
 vcov = vcov,
 sigma = sqrt(sigma2),
-df = df)
+df = df, p = p, q = q)
 }
 }
 else{
@@ -126,56 +172,79 @@ if(is.null(startval)){
 ## poisson regression to detect initial values for covariates
 pr<-optim(array(0.1,dim=c(1,k)),.poissonlik,y=y,x=x,method="BFGS",hessian=F)
 #arma regression to detect initial values for autoregressive parameters
-ar<-arma(y, order = c(1, 1))
-asval <- matrix(NA,1,3)
-asval[3]<-abs(coef(ar)[2])
-asval[2]<-abs(abs(coef(ar)[1])-asval[3])
-asval[1]<-(mean(y)*abs(1-asval[2]-asval[3]))
+ar<-arma(y, order = c(p, q))
+asval <- matrix(NA,1,1+p+q)
+for (i in 1:p){ asval[1+i] = abs(coef(ar)[i]) }
+for (i in 1:q){ asval[1+p+i] = abs(coef(ar)[p+i]) }
+asval[1]<-(mean(y)*abs(1-asval[2]-asval[1+p+1]))
 ## perform optimization
-p<-optim(c(pr$par,asval),.acplikc,y=y,x=x,method="BFGS",hessian=varopt)
+op<-optim(c(pr$par,asval),fn=.acplikc,gr=NULL,y=y,x=x,pp=p,qq=q,method="BFGS",hessian=varopt)
 }
 else{
-p<-optim(startval,.acplikc,y=y,x=x,method="BFGS",hessian=varopt)
+op<-optim(startval,fn=.acplikc,gr=NULL,y=y,x=x,pp=p,qq=q,method="BFGS",hessian=varopt)
 }
 ## compute coefficients
-coef<-p$par
+coef<-op$par
 ## compute conditional mean
 lambda <- matrix(NA,n,1)
-lambda[1] <- coef[k+1]+coef[k+2]*mean(y)+coef[k+3]*mean(y)
-for (t in 2:n) {lambda[t] <- coef[k+1]+coef[k+2]*y[t-1]+coef[k+3]*lambda[t-1]}
+lambda[1] <- mean(y)
+vylag = Lag(y,1)
+vparlag = c(coef[k+2])
+if(p > 1){
+for (i in 1:(p-1)){ vparlag <- append(vparlag, coef[k+2+i]) 
+vylag <- cbind(vylag,Lag(y,i+1))
+lambda[i+1] <- mean(y)
+}}
+vfilt = c(coef[k+2+p])
+vinit = c(lambda[1])
+if(q > 1){
+for (i in 1:(q-1)){ vfilt <- append(vfilt, coef[k+2+p+i]) 
+vinit <- append(vinit,lambda[1])
+}}
+lambda[(p+1):n] <- filter(coef[k+1] + na.omit(vylag%*%vparlag), filter = vfilt, init = vinit, method = "recursive")
 mu<-exp(x%*%coef[1:k])*lambda
 ## compute standardized residuals
 pres<-(y-mu)/sqrt(mu)
 ## degrees of freedom and standard deviation of residuals 
-df <- nrow(x)-ncol(x)-3
+df <- nrow(y)-k-p-q-1
 sigma2 <- sum(pres^2)/df
-if(is.null(p$hessian))
+if(is.null(op$hessian))
 {
-vcov<- matrix(NA,1,k+3)
+vcov<- matrix(NA,1,k+p+q+1)
 ## create vector of parameter names
-namevec <- matrix(NA,1,k+3)
+namevec <- matrix(NA,1,k+p+q+1)
 namevec[,1:k]<-colnames(x)
-namevec[,k+1]<-"a"
-namevec[,k+2]<-"b"
-namevec[,k+3]<-"c"
-list(coefficients = coef, vcov = vcov,
-df = df)
+namevec[k+1]<-"a"
+for (i in 1:p){ 
+namevec[k+1+i]<-paste("b",i)
+}
+for (i in 1:q){ 
+namevec[k+1+p+i]<-paste("c",i)
+}
+list(coefficients = coef,
+vcov = vcov,
+sigma = sqrt(sigma2),
+df = df, p = p, q = q)
 }
 else
 {
 ## compute covariance matrix
-vcov<-solve(p$hessian)
+vcov<-solve(op$hessian)
 ## create vector of parameter names
-namevec <- matrix(NA,1,k+3)
+namevec <- matrix(NA,1,k+p+q+1)
 namevec[,1:k]<-colnames(x)
-namevec[,k+1]<-"a"
-namevec[,k+2]<-"b"
-namevec[,k+3]<-"c"
+namevec[k+1]<-"a"
+for (i in 1:p){ 
+namevec[k+1+i]<-paste("b",i)
+}
+for (i in 1:q){ 
+namevec[k+1+p+i]<-paste("c",i)
+}
 colnames(vcov) <- rownames(vcov) <- namevec
 list(coefficients = coef,
 vcov = vcov,
 sigma = sqrt(sigma2),
-df = df)
+df = df, p = p, q = q)
 }
 }
 }
@@ -183,28 +252,53 @@ df = df)
 acp <- function(x, ...) UseMethod("acp")
 
 
-
-acp.default <- function(x, y, startval, varopt,...)
+acp.default <- function(x, y, p, q, startval, varopt,...)
 {
 n<-nrow(x)
 x <- as.matrix(x)
 y <- as.matrix(y)
 if(sum(x)==0){
 n<-nrow(x)
-est <- .acpEst(x, y ,startval, varopt)
+est <- .acpEst(x, y, p, q ,startval, varopt)
 lambda <- matrix(NA,n,1)
-lambda[1] <- est$coefficients[1]+est$coefficients[2]*mean(y)+est$coefficients[3]*mean(y)
-for (t in 2:n) {lambda[t] <- est$coefficients[1]+est$coefficients[2]*y[t-1]+est$coefficients[3]*lambda[t-1]}
+lambda[1] <- mean(y)
+vylag = Lag(y,1)
+vparlag = c(est$coefficients[2])
+if(p > 1){
+for (i in 1:(p-1)){ vparlag <- append(vparlag, est$coefficients[2+i]) 
+vylag <- cbind(vylag,Lag(y,i+1))
+lambda[i+1] <- mean(y)
+}}
+vfilt = c(est$coefficients[2+p])
+vinit = c(lambda[1])
+if(q > 1){
+for (i in 1:(q-1)){ vfilt <- append(vfilt, est$coefficients[2+p+i]) 
+vinit <- append(vinit,lambda[1])
+}}
+lambda[(p+1):n] <- filter(est$coefficients[1] + na.omit(vylag%*%vparlag), filter = vfilt, init = vinit, method = "recursive")
 mu<-lambda
 est$fitted.values <- as.vector(mu)
 est$residuals <- (y - est$fitted.values)/sqrt(est$fitted.values)
 }
 else{
 k<-ncol(x)
-est <- .acpEst(x, y ,startval, varopt)
+est <- .acpEst(x, y, p, q, startval, varopt)
 lambda <- matrix(NA,n,1)
-lambda[1] <- est$coefficients[k+1]+est$coefficients[k+2]*mean(y)+est$coefficients[k+3]*mean(y)
-for (t in 2:n) {lambda[t] <- est$coefficients[k+1]+est$coefficients[k+2]*y[t-1]+est$coefficients[k+3]*lambda[t-1]}
+lambda[1] <- mean(y)
+vylag = Lag(y,1)
+vparlag = c(est$coefficients[k+2])
+if(p > 1){
+for (i in 1:(p-1)){ vparlag <- append(vparlag, est$coefficients[k+2+i]) 
+vylag <- cbind(vylag,Lag(y,i+1))
+lambda[i+1] <- mean(y)
+}}
+vfilt = c(est$coefficients[k+2+p])
+vinit = c(lambda[1])
+if(q > 1){
+for (i in 1:(q-1)){ vfilt <- append(vfilt, est$coefficients[k+2+p+i]) 
+vinit <- append(vinit,lambda[1])
+}}
+lambda[(p+1):n] <- filter(est$coefficients[k+1] + na.omit(vylag%*%vparlag), filter = vfilt, init = vinit, method = "recursive")
 mu<-exp(x%*%est$coefficients[1:k])*lambda
 est$fitted.values <- as.vector(mu)
 est$residuals <- (y - est$fitted.values)/sqrt(est$fitted.values)
@@ -266,13 +360,13 @@ printCoefmat(x$coefficients, P.values=TRUE, has.Pvalue=TRUE)
 }
 
 
-acp.formula <- function(formula, data=list(), startval=NULL, varopt=T, family="acp",...)
+acp.formula <- function(formula, data=list(), p = NULL ,q = NULL ,startval=NULL, varopt=T, family="acp",...)
 {
 mf <- model.frame(formula=formula, data=data)
 x <- model.matrix(attr(mf, "terms"), data=mf)
 y <- model.response(mf)
 if (family=="acp"){
-est <- acp.default(x, y, startval, varopt,...)
+est <- acp.default(x, y, p, q, startval, varopt,...)
 }
 else{
 est <- .poisson.default(x, y, ...)
@@ -280,128 +374,44 @@ est <- .poisson.default(x, y, ...)
 est$call <- match.call()
 est$formula <- formula
 est$data <- data
+est$family <- family
 est
 }
 
-predict.acp <- function(object, newxdata=NULL, newydata=NULL,...)
+predict.acp <- function(object, newydata=NULL , newxdata=NULL,...)
 {
-if(is.null(newxdata))
-y <- fitted(object)
-else{
-if(!is.null(object$formula)){
-## model has been fitted using formula interface
-x <- model.matrix(object$formula, newxdata)
-}
-else{
-x <- newxdata
-}
-if(sum(x)==0){
-n<-nrow(x)
+if(is.null(newydata)){y <- fitted(object)}
+n<-length(newydata)
+k<-ifelse(is.null(newxdata),0,ncol(newxdata))
+if(k>0){x<-newxdata}
+if (object$family=="acp"){
+q = object$q
+p = object$p
 lambda <- matrix(NA,n,1)
 lambda[1] <- 1
-for (t in 2:n) {lambda[t] <- coef(object)[1]+coef(object)[2]*newydata[t-1]+coef(object)[3]*lambda[t-1]}
-y <- lambda
+vylag = Lag(newydata,1)
+vparlag = c(coef(object)[k+2])
+if(p > 1){
+for (i in 1:(p-1)){ vparlag <- append(vparlag, coef(object)[k+2+i]) 
+vylag <- cbind(vylag,Lag(newydata,i+1))
+lambda[i+1] <- 1
+}}
+vfilt = c(coef(object)[k+2+p])
+vinit = c(lambda[1])
+if(q > 1){
+for (i in 1:(q-1)){ vfilt <- append(vfilt, coef(object)[k+2+p+i]) 
+vinit <- append(vinit,lambda[1])
+}}
+lambda[(p+1):n] <- filter(coef(object)[k+1] +  na.omit(vylag%*%vparlag), filter = vfilt, init = vinit, method = "recursive")
+if (k==0) {
+   xb=matrix(1,n,1)
+} else {
+   xb=exp(as.matrix(x)%*%coef(object)[1:k])
+}
+y <- as.vector(xb*lambda)
 }
 else{
-n<-nrow(x)
-k<-ncol(x)
-
-if(sum(x[,1])==nrow(x)){
-y <-as.vector(exp(x %*% coef(object)))
-}
-else{
-lambda <- matrix(NA,n,1)
-lambda[1] <- 1
-for (t in 2:n) {lambda[t] <- coef(object)[k+1]+coef(object)[k+2]*newydata[t-1]+coef(object)[k+3]*lambda[t-1]}
-y <- as.vector(exp(x%*%coef(object)[1:k])*lambda)
-}
-}
-}
-y
-}
-
-forecast <- function(object, sample, ydata,...)
-{
-newxdata<-object$data
-x<-model.matrix(object$formula, newxdata)
-n<-nrow(x)
-if(sum(x)==0){
-
-vpar<-matrix(NA,3,(n-sample+1))
-fordata<-newxdata[c(1:(sample)),]
-modfor <- acp(object$formula,data=fordata,varopt=F)
-vpar[,1]<-coef(modfor)
-lambdatempv<-matrix(NA,n-sample,1)
-for (t in 1:(n-sample)){
-fordata<-newxdata[c(1:(sample+t)),]
-modfor <- acp(object$formula,data=fordata,coef(modfor),varopt=F)
-vpar[,t+1]<-coef(modfor)
-lambdatemp <- matrix(NA,sample+t,1)
-lambdatemp[1] <- mean(ydata)
-for (tt in 2:(sample+t)) {lambdatemp[tt] <- coef(modfor)[1]+coef(modfor)[2]*ydata[tt-1]+coef(modfor)[3]*lambdatemp[tt-1]}
-lambdatempv[t]<-lambdatemp[sample+t]
-}
-lambda <- matrix(NA,n-sample,1)
-lambda[1]<- mean(ydata)
-for (tttt in 2:(n-sample)) {
-lambda[tttt] <- vpar[1,tttt]+vpar[2,tttt]*ydata[sample+tttt-1]+vpar[3,tttt]*lambdatempv[tttt-1]
-}
-y <- as.vector(lambda)
-
-}
-else{
-
-if(sum(x[,1])==nrow(x)){
-
-k<-ncol(x)
-vpar<-matrix(NA,k,(n-sample+1))
-fordata<-newxdata[c(1:(sample)),]
-modfor <- acp(object$formula,data=fordata,varopt=F,family="poisson")
-vpar[,1]<-coef(modfor)
-for (t in 1:(n-sample)){
-fordata<-newxdata[c(1:(sample+t)),]
-modfor <- acp(object$formula,data=fordata,coef(modfor),varopt=F,family="poisson")
-vpar[,t+1]<-coef(modfor)
-}
-vexpbf<-matrix(NA,n-sample,1)
-for (ttt in 1:(n-sample)) {
-expbf<-exp(vpar[1:k,ttt]%*%x[sample+ttt,])
-vexpbf[ttt]<-expbf
-}
-y <- as.vector(vexpbf)
-
-}
-else{
-
-k<-ncol(x)
-vpar<-matrix(NA,k+3,(n-sample+1))
-fordata<-newxdata[c(1:(sample)),]
-modfor <- acp(object$formula,data=fordata,varopt=F)
-vpar[,1]<-coef(modfor)
-lambdatempv<-matrix(NA,n-sample,1)
-for (t in 1:(n-sample)){
-fordata<-newxdata[c(1:(sample+t)),]
-modfor <- acp(object$formula,data=fordata,coef(modfor),varopt=F)
-vpar[,t+1]<-coef(modfor)
-lambdatemp <- matrix(NA,sample+t,1)
-lambdatemp[1] <- mean(ydata)
-for (tt in 2:(sample+t)) {lambdatemp[tt] <- coef(modfor)[k+1]+coef(modfor)[k+2]*ydata[tt-1]+coef(modfor)[k+3]*lambdatemp[tt-1]}
-lambdatempv[t]<-lambdatemp[sample+t]
-}
-vexpbf<-matrix(NA,n-sample,1)
-for (ttt in 1:(n-sample)) {
-expbf<-exp(vpar[1:k,ttt]%*%x[sample+ttt,])
-vexpbf[ttt]<-expbf
-}
-lambda <- matrix(NA,n-sample,1)
-lambda[1]<- 1
-for (tttt in 2:(n-sample)) {
-lambda[tttt] <- vpar[k+1,tttt]+vpar[k+2,tttt]*ydata[sample+tttt-1]+vpar[k+3,tttt]*lambdatempv[tttt-1]
-}
-y <- as.vector(lambda*vexpbf)
-
-}
-
+y <- as.vector(exp(as.matrix(x)%*%coef(object)[1:k]))
 }
 y
 }
@@ -524,7 +534,3 @@ berkowitz <- as.table(berkowitz)
 berkowitz
 }
 
-
-
-
-
